# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Tailbone Provider for CORE-POS Integration
"""

from sqlalchemy.orm import sessionmaker, scoped_session
from zope.sqlalchemy import register

from tailbone.providers import TailboneProvider


class TailboneCorePosProvider(TailboneProvider):
    """
    View provider for tailbone-corepos
    """
    key = 'tailbone_corepos'

    def configure_db_sessions(self, rattail_config, pyramid_config):
        from tailbone_corepos.db import (CoreOfficeSession, 
                                         CoreTransSession,
                                         CoreTransArchiveSession,
                                         ExtraCoreOfficeSessions,
                                         ExtraCoreTransSessions,
                                         ExtraCoreTransArchiveSessions)

        # CORE-POS DB(s)
        CoreOfficeSession.configure(bind=rattail_config.core_office_op_engine)
        CoreTransSession.configure(bind=rattail_config.core_office_trans_engine)
        CoreTransArchiveSession.configure(
            bind=rattail_config.core_office_trans_archive_engine)

        # create session wrappers for each "extra" CORE DB engine
        for key, engine in rattail_config.core_office_op_engines.items():
            if key != 'default':
                Session = scoped_session(sessionmaker(bind=engine))
                register(Session)
                ExtraCoreOfficeSessions[key] = Session

        # create session wrappers for each "extra" CORE Transaction DB engine
        for key, engine in rattail_config.core_office_trans_engines.items():
            if key != 'default':
                Session = scoped_session(sessionmaker(bind=engine))
                register(Session)
                ExtraCoreTransSessions[key] = Session

        # and same for CORE Transaction Archive DB engine(s)
        for key, engine in rattail_config.core_office_trans_archive_engines.items():
            if key != 'default':
                Session = scoped_session(sessionmaker(bind=engine))
                register(Session)
                ExtraCoreTransArchiveSessions[key] = Session

    def get_provided_views(self):
        return {

            'corepos': {

                'people': {
                    'tailbone_corepos.views.corepos.customers': {
                        'label': "Customers",
                    },
                    'tailbone_corepos.views.corepos.employees': {
                        'label': "Employees",
                    },
                    'tailbone_corepos.views.corepos.groups': {
                        'label': "User Groups",
                    },
                    'tailbone_corepos.views.corepos.members': {
                        'label': "Members",
                    },
                    'tailbone_corepos.views.corepos.users': {
                        'label': "Users",
                    },
                },

                'products': {
                    'tailbone_corepos.views.corepos.departments': {
                        'label': "Departments",
                    },
                    'tailbone_corepos.views.corepos.subdepartments': {
                        'label': "Subdepartments",
                    },
                    'tailbone_corepos.views.corepos.superdepartments': {
                        'label': "Super Departments",
                    },
                    'tailbone_corepos.views.corepos.likecodes': {
                        'label': "Like Codes",
                    },
                    'tailbone_corepos.views.corepos.origins': {
                        'label': "Origins",
                    },
                    'tailbone_corepos.views.corepos.products': {
                        'label': "Products",
                    },
                    'tailbone_corepos.views.corepos.scaleitems': {
                        'label': "Scale Items",
                    },
                    'tailbone_corepos.views.corepos.vendoritems': {
                        'label': "Vendor Items",
                    },
                    'tailbone_corepos.views.corepos.vendors': {
                        'label': "Vendors",
                    },
                },

                'other': {
                    'tailbone_corepos.views.corepos.batches': {
                        'label': "Batches",
                    },
                    'tailbone_corepos.views.corepos.coupons': {
                        'label': "Coupons",
                    },
                    'tailbone_corepos.views.corepos.parameters': {
                        'label': "Parameters",
                    },
                    'tailbone_corepos.views.corepos.purchaseorders': {
                        'label': "Purchase Orders",
                    },
                    'tailbone_corepos.views.corepos.stores': {
                        'label': "Stores",
                    },
                    'tailbone_corepos.views.corepos.tablesyncrules': {
                        'label': "Table Sync Rules",
                    },
                    'tailbone_corepos.views.corepos.taxrates': {
                        'label': "Tax Rates",
                    },
                    'tailbone_corepos.views.corepos.transactions': {
                        'label': "Transactions",
                    },
                },
            },
        }

    def make_integration_menu(self, request, **kwargs):
        from tailbone_corepos.menus import make_corepos_menu
        return make_corepos_menu(request)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS employee views
"""

from corepos.db.office_op import model as corepos

from .master import CoreOfficeMasterView


class EmployeeView(CoreOfficeMasterView):
    """
    Base class for employee views.
    """
    model_class = corepos.Employee
    model_title = "CORE-POS Employee"
    url_prefix = '/core-pos/employees'
    route_prefix = 'corepos.employees'

    grid_columns = [
        'number',
        'first_name',
        'last_name',
        'job_title',
        'active',
        'birth_date',
    ]

    def configure_grid(self, g):
        super(EmployeeView, self).configure_grid(g)

        g.filters['active'].default_active = True
        g.filters['active'].default_verb = 'is_true'

        g.filters['first_name'].default_active = True
        g.filters['first_name'].default_verb = 'contains'

        g.filters['last_name'].default_active = True
        g.filters['last_name'].default_verb = 'contains'

        g.set_sort_defaults('number')

        g.set_link('number')
        g.set_link('first_name')
        g.set_link('last_name')

    def grid_extra_class(self, employee, i):
        if not employee.active:
            return 'warning'

    def configure_form(self, f):
        super(EmployeeView, self).configure_form(f)

        # TODO: should expose this as date+time widget?
        f.set_readonly('birth_date')

    def core_office_object_url(self, office_url, employee):
        return '{}/admin/Cashiers/CashierEditor.php?emp_no={}'.format(
            office_url, employee.number)


def defaults(config, **kwargs):
    base = globals()

    EmployeeView = kwargs.get('EmployeeView', base['EmployeeView'])
    EmployeeView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS member views
"""

from sqlalchemy import orm

from corepos.db.office_op import model as corepos
from corepos.db.office_trans import model as coretrans

from webhelpers2.html import HTML, tags

from tailbone_corepos.db import CoreTransSession
from .master import CoreOfficeMasterView


class MemberTypeView(CoreOfficeMasterView):
    """
    Master view for member types
    """
    model_class = corepos.MemberType
    model_title = "CORE-POS Member Type"
    url_prefix = '/core-pos/member-types'
    route_prefix = 'corepos.member_types'

    labels = {
        'id': "ID",
        'ssi': "SSI",
    }

    def configure_grid(self, g):
        super().configure_grid(g)

        g.set_link('id')
        g.set_link('description')


class MemberView(CoreOfficeMasterView):
    """
    Master view for members
    """
    model_class = corepos.MemberInfo
    model_title = "CORE-POS Member (classic)"
    model_title_plural = "CORE-POS Members (classic)"
    url_prefix = '/core-pos/members'
    route_prefix = 'corepos.members'

    labels = {
        'card_number': "Card No.",
        'email2': "Email 2",
        'ads_ok': "Ads OK",
    }

    grid_columns = [
        'card_number',
        'first_name',
        'last_name',
        'street',
        'city',
        'state',
        'zip',
        'phone',
        'email',
    ]

    form_fields = [
        'card_number',
        'first_name',
        'last_name',
        'other_first_name',
        'other_last_name',
        'customers',
        'street',
        'city',
        'state',
        'zip',
        'phone',
        'email',
        'email2',
        'ads_ok',
        'start_date',
        'end_date',
        'barcodes',
        'suspension',
        'equity_live_balance',
    ]

    has_rows = True
    model_row_class = coretrans.StockPurchase
    rows_title = "Stock Purchases"

    row_labels = {
        'transaction_id': "Transaction ID",
    }

    row_grid_columns = [
        'datetime',
        'amount',
        'transaction_number',
        'transaction_id',
        'department_number',
    ]

    def configure_grid(self, g):
        super().configure_grid(g)

        g.set_sort_defaults('card_number')

        g.filters['card_number'].default_active = True
        g.filters['card_number'].default_verb = 'equal'

        g.set_link('card_number')
        g.set_link('first_name')
        g.set_link('last_name')

    def configure_form(self, f):
        super().configure_form(f)

        # dates
        f.set_renderer('start_date', self.render_member_date)
        f.set_renderer('end_date', self.render_member_date)

        f.set_renderer('barcodes', self.render_barcodes)

        f.set_readonly('customers')
        f.set_renderer('customers', self.render_customers)

        f.set_renderer('suspension', self.render_suspension)

        f.set_renderer('equity_live_balance', self.render_equity_live_balance)

    def render_member_date(self, member, field):
        date = getattr(member.dates, field)
        if date:
            return str(date.date())

    def render_barcodes(self, member, field):
        barcodes = member.barcodes
        if not barcodes:
            return

        items = []
        for barcode in barcodes:
            if barcode.upc:
                text = barcode.upc
            elif barcode.upc is None:
                text = "(null)"
            else:
                text = "(empty string)"
            items.append(HTML.tag('li', c=[text]))
        return HTML.tag('ul', c=items)

    def render_customers(self, member, field):
        customers = member.customers
        if not customers:
            return

        items = []
        for customer in customers:
            text = str(customer)
            url = self.request.route_url('corepos.customers.view',
                                         id=customer.id)
            link = tags.link_to(text, url)
            items.append(HTML.tag('li', c=[link]))
        return HTML.tag('ul', c=items)

    def render_suspension(self, member, field):
        suspension = member.suspension
        if not suspension:
            return

        text = str(suspension)
        url = self.request.route_url('corepos.suspensions.view',
                                     card_number=suspension.card_number)
        return tags.link_to(text, url)

    def render_equity_live_balance(self, member, field):
        try:
            balance = CoreTransSession.query(coretrans.EquityLiveBalance)\
                                      .filter(coretrans.EquityLiveBalance.member_number == member.card_number)\
                                      .one()
        except orm.exc.NoResultFound:
            return

        app = self.get_rattail_app()
        return app.render_currency(balance.payments)

    def get_xref_buttons(self, member):
        app = self.get_rattail_app()
        corepos = app.get_corepos_handler()
        url = corepos.get_office_member_url(member.card_number)
        if url:
            return [self.make_xref_button(url=url, text="View in CORE Office")]

    def get_row_data(self, member):
        return CoreTransSession.query(coretrans.StockPurchase)\
                               .filter(coretrans.StockPurchase.card_number == member.card_number)

    def get_parent(self, stock_purchase):
        return self.Session.get(corepos.MemberInfo, stock_purchase.card_number)

    def configure_row_grid(self, g):
        super().configure_row_grid(g)

        g.set_type('amount', 'currency')

        g.set_sort_defaults('datetime', 'desc')

    def row_view_action_url(self, stock_purchase, i):
        return self.request.route_url('corepos.stock_purchases.view',
                                      card_number=stock_purchase.card_number,
                                      datetime=stock_purchase.datetime,
                                      transaction_number=stock_purchase.transaction_number,
                                      department_number=stock_purchase.department_number)


def defaults(config, **kwargs):
    base = globals()

    MemberTypeView = kwargs.get('MemberTypeView', base['MemberTypeView'])
    MemberTypeView.defaults(config)

    MemberView = kwargs.get('MemberView', base['MemberView'])
    MemberView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE POS tender views
"""

from corepos.db.office_op.model import Tender

from .master import CoreOfficeMasterView


class TenderView(CoreOfficeMasterView):
    """
    Master view for tenders
    """
    model_class = Tender
    model_title = "CORE-POS Tender"
    url_prefix = '/core-pos/tenders'
    route_prefix = 'corepos.tenders'

    labels = {
        'tender_id': "Tender ID",
    }

    def configure_grid(self, g):
        super().configure_grid(g)

        g.set_link('tender_id')
        g.set_sort_defaults('tender_id')

        g.set_link('tender_name')

        g.set_type('min_amount', 'currency')
        g.set_type('max_amount', 'currency')
        g.set_type('max_refund', 'currency')


def defaults(config, **kwargs):
    base = globals()

    TenderView = kwargs.get('TenderView', base['TenderView'])
    TenderView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS transaction views
"""

from collections import OrderedDict

from corepos.db.office_trans import model as coretrans

from rattail_corepos.corepos.office.importing.db.square import FromSquareToCoreTrans

from .base import TransactionDetailMasterView
from tailbone_corepos.db import CoreTransSession, ExtraCoreTransSessions


class TransactionDetailView(TransactionDetailMasterView):
    """
    Master view for "current" transaction details.
    """
    model_class = coretrans.TransactionDetail
    model_title = "CORE-POS Current Transaction Detail"
    url_prefix = '/corepos/transaction-details/current'
    route_prefix = 'corepos.transaction_details'
    bulk_deletable = True
    supports_import_batch_from_file = True

    @property
    def Session(self):
        """
        Which session we return will depend on user's "current" engine.
        """
        dbkey = self.get_current_engine_dbkey()

        if dbkey != 'default' and dbkey in ExtraCoreTransSessions:
            return ExtraCoreTransSessions[dbkey]

        return CoreTransSession

    def get_db_engines(self):
        engines = OrderedDict()
        if self.rattail_config.core_office_trans_engine:
            engines['default'] = self.rattail_config.core_office_trans_engine
        for dbkey in self.rattail_config.core_office_trans_engines:
            if dbkey != 'default':
                engines[dbkey] = self.rattail_config.core_office_trans_engines[dbkey]
        return engines

    def make_isolated_session(self):
        from corepos.db.office_trans import Session as CoreTransSession

        dbkey = self.get_current_engine_dbkey()
        if dbkey != 'default' and dbkey in self.rattail_config.core_office_trans_engines:
            return CoreTransSession(bind=self.rattail_config.core_office_trans_engines[dbkey])

        return CoreTransSession()

    def import_square(self):
        return self.import_batch_from_file(FromSquareToCoreTrans, 'TransactionDetail',
                                           importer_host_title="Square CSV")

    @classmethod
    def defaults(cls, config):
        route_prefix = cls.get_route_prefix()
        url_prefix = cls.get_url_prefix()
        permission_prefix = cls.get_permission_prefix()

        # import from square
        config.add_route('{}.import_square'.format(route_prefix), '{}/import-square'.format(url_prefix))
        config.add_view(cls, attr='import_square', route_name='{}.import_square'.format(route_prefix),
                        permission='{}.import_file'.format(permission_prefix))

        cls._defaults(config)


def defaults(config, **kwargs):
    base = globals()

    TransactionDetailView = kwargs.get('TransactionDetailView', base['TransactionDetailView'])
    TransactionDetailView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Views for CORE-POS Stock Purchases
"""

from corepos.db.office_trans import model as corepos

from .master import CoreTransMasterView


class StockPurchaseView(CoreTransMasterView):
    """
    Master view for tax rates
    """
    model_class = corepos.StockPurchase
    model_title = "CORE-POS Stock Purchase"
    url_prefix = '/core-pos/stock-purchases'
    route_prefix = 'corepos.stock_purchases'
    supports_grid_totals = True

    labels = {
        'transaction_id': "Transaction ID",
    }

    def configure_grid(self, g):
        super().configure_grid(g)

        g.set_type('amount', 'currency')

        g.set_sort_defaults('datetime', 'desc')

        g.set_link('card_number')
        g.set_link('transaction_number')
        g.set_link('datetime')

    def fetch_grid_totals(self):
        app = self.get_rattail_app()
        results = self.get_effective_data()
        total = sum([purchase.amount for purchase in results])
        return {'totals_display': app.render_currency(total)}

    def configure_form(self, f):
        super().configure_form(f)

        f.set_renderer('card_number', self.render_linked_corepos_card_number)

        f.set_type('amount', 'currency')

        f.set_renderer('department_number',
                       self.render_linked_corepos_department_number)


def defaults(config, **kwargs):
    base = globals()

    StockPurchaseView = kwargs.get('StockPurchaseView', base['StockPurchaseView'])
    StockPurchaseView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE POS master view
"""

from collections import OrderedDict

from webhelpers2.html import tags

from tailbone.views import MasterView
from tailbone_corepos.db import (CoreOfficeSession, ExtraCoreOfficeSessions,
                                 CoreTransSession)


class CoreMasterView(MasterView):
    """
    Master base class for all CORE-POS views
    """
    supports_multiple_engines = True
    has_local_times = True
    results_downloadable = True

    labels = {
        'id': "ID",
    }

    def render_local_date(self, obj, field):
        value = getattr(obj, field)
        if not value:
            return ""
        app = self.get_rattail_app()
        value = app.localtime(value)
        return str(value.date())

    def render_linked_corepos_card_number(self, obj, field):
        card_number = getattr(obj, field)
        if not card_number:
            return

        text = str(card_number)
        url = self.request.route_url('corepos.members.view',
                                     card_number=card_number)
        return tags.link_to(text, url)

    def render_corepos_store(self, obj, field):
        store = getattr(obj, field)
        if not store:
            return ""
        text = "({}) {}".format(store.id, store.description)
        url = self.request.route_url('corepos.stores.view', id=store.id)
        return tags.link_to(text, url)

    def render_corepos_department(self, obj, field):
        department = getattr(obj, field)
        if not department:
            return ""
        text = "({}) {}".format(department.number, department.name)
        url = self.request.route_url('corepos.departments.view', number=department.number)
        return tags.link_to(text, url)

    def render_linked_corepos_department_number(self, obj, field):
        department_number = getattr(obj, field)
        if not department_number:
            return

        text = str(department_number)
        url = self.request.route_url('corepos.departments.view',
                                     number=department_number)
        return tags.link_to(text, url)

    def render_corepos_tax_rate(self, obj, field):
        taxrate = getattr(obj, field)
        if not taxrate:
            return
        text = str(taxrate)
        url = self.request.route_url('corepos.taxrates.view', id=taxrate.id)
        return tags.link_to(text, url)

    def render_corepos_vendor(self, obj, field):
        vendor = getattr(obj, field)
        if not vendor:
            return ""
        text = "({}) {}".format(vendor.abbreviation, vendor.name)
        url = self.request.route_url('corepos.vendors.view', id=vendor.id)
        return tags.link_to(text, url)

    def render_corepos_product(self, obj, field):
        product = getattr(obj, field)
        if not product:
            return ""
        text = str(product)
        url = self.request.route_url('corepos.products.view', id=product.id)
        return tags.link_to(text, url)

    def template_kwargs_view(self, **kwargs):
        """
        Adds the URL for viewing the record/object within CORE Office, or else
        the reason for lack of such a URL.
        """
        kwargs = super().template_kwargs_view(**kwargs)
        app = self.get_rattail_app()
        corepos = app.get_corepos_handler()
        obj = kwargs['instance']

        # CORE Office URL
        kwargs['core_office_url'] = None
        office_url = corepos.get_office_url()
        if not office_url:
            kwargs['core_office_why_no_url'] = "CORE Office URL is not configured"
        else:
            url = self.core_office_object_url(office_url, obj)
            if url:
                kwargs['core_office_url'] = url
            else:
                kwargs['core_office_why_no_url'] = "URL not defined for this object"

        return kwargs

    def core_office_object_url(self, office_url, obj):
        """
        Subclass must define this logic; should return the "final" CORE Office
        URL for the given object.
        """


class CoreOfficeMasterView(CoreMasterView):
    """
    Master base class for CORE Office views
    """
    engine_type_key = 'corepos'

    def get_db_engines(self):
        engines = OrderedDict()
        if self.rattail_config.core_office_op_engine:
            engines['default'] = self.rattail_config.core_office_op_engine
        for dbkey in self.rattail_config.core_office_op_engines:
            if dbkey != 'default':
                engines[dbkey] = self.rattail_config.core_office_op_engines[dbkey]
        return engines

    @property
    def Session(self):
        """
        Which session we return will depend on user's "current" engine.
        """
        dbkey = self.get_current_engine_dbkey()

        if dbkey != 'default' and dbkey in ExtraCoreOfficeSessions:
            return ExtraCoreOfficeSessions[dbkey]

        return CoreOfficeSession

    def make_isolated_session(self):
        from corepos.db.office_op import Session as CoreSession

        dbkey = self.get_current_engine_dbkey()
        if dbkey != 'default' and dbkey in self.rattail_config.corepos_engines:
            return CoreSession(bind=self.rattail_config.corepos_engines[dbkey])

        return CoreSession()


class CoreTransMasterView(CoreMasterView):
    """
    Master base class for CORE Office "trans" DB views
    """

    @property
    def Session(self):
        return CoreTransSession



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS store views
"""

from corepos.db.office_op import model as corepos

from .master import CoreOfficeMasterView


class StoreView(CoreOfficeMasterView):
    """
    Base class for store views.
    """
    model_class = corepos.Store
    model_title = "CORE-POS Store"
    url_prefix = '/core-pos/stores'
    route_prefix = 'corepos.stores'

    labels = {
        'db_host': "DB Host",
        'db_driver': "DB Driver",
        'db_user': "DB User",
        'db_password': "DB Password",
        'trans_db': "Transactions DB",
        'op_db': "Operational DB",
        'web_service_url': "Web Service URL",
    }

    grid_columns = [
        'id',
        'description',
        'push',
        'pull',
        'has_own_items',
    ]

    form_fields = [
        'id',
        'description',
        'db_host',
        'db_driver',
        # TODO: should maybe expose these for admin only?
        # 'db_user',
        # 'db_password',
        'trans_db',
        'op_db',
        'push',
        'pull',
        'has_own_items',
        'web_service_url',
    ]

    def configure_grid(self, g):
        super(StoreView, self).configure_grid(g)

        # must add this b/c id is technically just a synonym
        g.set_sorter('id', corepos.Store.storeID)

        g.set_sort_defaults('id')

        g.set_link('id')
        g.set_link('description')


def defaults(config, **kwargs):
    base = globals()

    StoreView = kwargs.get('StoreView', base['StoreView'])
    StoreView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE POS customer views
"""

from corepos.db.office_op import model as corepos

from webhelpers2.html import tags

from .master import CoreOfficeMasterView


def render_member_info(request, custdata, field):
    meminfo = custdata.member_info
    if not meminfo:
        return
    text = str(meminfo)
    url = request.route_url('corepos.members.view',
                            card_number=meminfo.card_number)
    return tags.link_to(text, url)


class CustomerClassicView(CoreOfficeMasterView):
    """
    Master view for "classic" customers, i.e. ``custdata`` table
    """
    model_class = corepos.CustomerClassic
    model_title = "CORE-POS Customer (classic)"
    model_title_plural = "CORE-POS Customers (classic)"
    url_prefix = '/core-pos/customers'
    route_prefix = 'corepos.customers'
    results_downloadable = True
    set_deletable = True
    mergeable = True

    labels = {
        'id': "ID",
        'card_number': "Card No.",
        'person_number': "Person No.",
        'charge_ok': "Charge OK",
        'member_type_id': "Member Type No.",
        'number_of_checks': "Number of Checks",
        'ssi': "SSI",
    }

    grid_columns = [
        'card_number',
        'person_number',
        'first_name',
        'last_name',
        'member_type',
        'ssi',
        'charge_ok',
        'charge_limit',
        'balance',
        'write_checks',
        'purchases',
    ]

    def query(self, session):
        query = super().query(session)

        query = query.outerjoin(corepos.MemberInfo,
                                corepos.MemberInfo.card_number == corepos.CustomerClassic.card_number)

        return query

    def configure_grid(self, g):
        super().configure_grid(g)

        g.filters['card_number'].default_active = True
        g.filters['card_number'].default_verb = 'equal'

        g.filters['first_name'].default_active = True
        g.filters['first_name'].default_verb = 'contains'

        g.filters['last_name'].default_active = True
        g.filters['last_name'].default_verb = 'contains'

        g.set_joiner('member_type_description', lambda q: q.outerjoin(corepos.MemberType))
        g.set_sorter('member_type_description', corepos.MemberType.description)
        g.set_filter('member_type_description', corepos.MemberType.description)

        g.set_filter('member_info_email', corepos.MemberInfo.email)
        g.set_filter('member_info_email2', corepos.MemberInfo.email2)

        g.set_type('charge_limit', 'currency')
        g.set_type('balance', 'currency')
        g.set_type('purchases', 'currency')

        g.set_sort_defaults('card_number')

        g.set_link('card_number')
        g.set_link('first_name')
        g.set_link('last_name')

    def get_uuid_for_grid_row(self, customer):
        # nb. uniquely identify records, for merge support
        return customer.id

    def configure_form(self, f):
        super().configure_form(f)

        f.set_renderer('member_info', self.render_member_info)
        f.set_renderer('member_type', self.render_member_type)

        if self.creating or self.editing:
            f.remove_field('member_info')
            f.remove_field('member_type')
            f.remove_field('last_change')
        else:
            f.set_type('last_change', 'datetime_local')

    def render_member_type(self, custdata, field):
        memtype = custdata.member_type
        if not memtype:
            return
        text = str(memtype)
        url = self.request.route_url('corepos.member_types.view', id=memtype.id)
        return tags.link_to(text, url)

    def render_member_info(self, custdata, field):
        return render_member_info(self.request, custdata, field)

    def core_office_object_url(self, office_url, customer):
        app = self.get_rattail_app()
        corepos = app.get_corepos_handler()
        return corepos.get_office_member_url(customer.card_number)

    def download_results_fields_available(self, **kwargs):
        fields = super().download_results_fields_available(**kwargs)

        fields.extend([
            'email',
            'email2',
        ])

        return fields

    def download_results_normalize(self, custdata, fields, **kwargs):
        data = super().download_results_normalize(custdata, fields, **kwargs)

        # import ipdb; ipdb.set_trace()

        for field in ('email', 'email2'):
            if field in fields:
                data[field] = getattr(custdata.member_info, field) if custdata.member_info else None

        return data


class CustomerAccountView(CoreOfficeMasterView):
    """
    Master view for "new" customer accounts, i.e. ``CustomerAccounts``
    table
    """
    model_class = corepos.CustomerAccount
    model_title = "CORE-POS Customer Account (new)"
    model_title_plural = "CORE-POS Customer Accounts (new)"
    url_prefix = '/core-pos/customer-accounts-new'
    route_prefix = 'corepos.customer_accounts_new'
    results_downloadable = True

    grid_columns = [
        'id',
        'card_number',
        'member_status',
        'active_status',
        'customer_type',
        'start_date',
        'end_date',
        'modified',
    ]

    def configure_grid(self, g):
        super().configure_grid(g)

        g.filters['card_number'].default_active = True
        g.filters['card_number'].default_verb = 'equal'
        g.set_sort_defaults('card_number')


class CustomerView(CoreOfficeMasterView):
    """
    Master view for "new" customers, i.e. ``Customers`` table
    """
    model_class = corepos.Customer
    model_title = "CORE-POS Customer (new)"
    model_title_plural = "CORE-POS Customers (new)"
    url_prefix = '/core-pos/customers-new'
    route_prefix = 'corepos.customers_new'
    results_downloadable = True

    labels = {
        'account_id': "Account ID",
    }

    grid_columns = [
        'id',
        'account',
        'card_number',
        'first_name',
        'last_name',
        'account_holder',
        'staff',
        'modified',
    ]

    def configure_grid(self, g):
        super().configure_grid(g)

        g.filters['card_number'].default_active = True
        g.filters['card_number'].default_verb = 'equal'
        g.set_sort_defaults('card_number')


class CustomerNotificationView(CoreOfficeMasterView):
    """
    Master view for customers notifications
    """
    model_class = corepos.CustomerNotification
    model_title = "CORE-POS Customer Notification"
    model_title_plural = "CORE-POS Customer Notifications"
    url_prefix = '/core-pos/customer-notifications'
    route_prefix = 'corepos.customer_notifications'

    labels = {
        'customer_id': "Customer ID",
    }

    def configure_grid(self, g):
        super().configure_grid(g)

        g.set_link('id')
        g.set_sort_defaults('id', 'desc')

        g.set_link('card_number')
        if 'card_number' in g.filters:
            g.filters['card_number'].default_active = True
            g.filters['card_number'].default_verb = 'equal'

        g.set_link('message')


class SuspensionView(CoreOfficeMasterView):
    """
    Master view for legacy customer suspensions.
    """
    model_class = corepos.Suspension
    model_title = "CORE-POS Suspension"
    model_title_plural = "CORE-POS Suspensions"
    url_prefix = '/core-pos/suspensions'
    route_prefix = 'corepos.suspensions'

    labels = {
        'reason_object': "Reason Code",
    }

    grid_columns = [
        'card_number',
        'type',
        'memtype1',
        'memtype2',
        'suspension_date',
        'reason',
        'mail_flag',
        'discount',
        'charge_limit',
        'reason_object',
    ]

    form_fields = [
        'card_number',
        'member_info',
        'type',
        'memtype1',
        'memtype2',
        'suspension_date',
        'reason',
        'mail_flag',
        'discount',
        'charge_limit',
        'reason_object',
    ]

    def configure_grid(self, g):
        super(SuspensionView, self).configure_grid(g)

        g.filters['card_number'].default_active = True
        g.filters['card_number'].default_verb = 'equal'

        g.set_type('charge_limit', 'currency')

        g.set_sort_defaults('card_number')

        g.set_link('card_number')

    def configure_form(self, f):
        super(SuspensionView, self).configure_form(f)

        f.set_renderer('member_info', self.render_member_info)

        f.set_type('charge_limit', 'currency')

    def render_member_info(self, custdata, field):
        return render_member_info(self.request, custdata, field)


def defaults(config, **kwargs):
    base = globals()

    CustomerClassicView = kwargs.get('CustomerClassicView', base['CustomerClassicView'])
    CustomerClassicView.defaults(config)

    CustomerAccountView = kwargs.get('CustomerAccountView', base['CustomerAccountView'])
    CustomerAccountView.defaults(config)

    CustomerView = kwargs.get('CustomerView', base['CustomerView'])
    CustomerView.defaults(config)

    CustomerNotificationView = kwargs.get('CustomerNotificationView', base['CustomerNotificationView'])
    CustomerNotificationView.defaults(config)

    SuspensionView = kwargs.get('SuspensionView', base['SuspensionView'])
    SuspensionView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE POS "lane" views
"""

from rattail_corepos.corepos.office.util import get_fannie_config_value

from .master import CoreOfficeMasterView


class LaneView(CoreOfficeMasterView):
    """
    Master view for CORE lanes
    """
    model_key = 'number'
    model_title = "CORE-POS Lane"
    url_prefix = '/core-pos/lanes'
    route_prefix = 'corepos.lanes'
    filterable = False
    pageable = False
    creatable = False
    viewable = False
    editable = False
    deletable = False
    results_downloadable = False

    grid_columns = [
        'number',
        'host',
        'type',
        'op',
        'trans',
        'offline',
    ]

    def get_data(self, session=None):
        data = []
        lanes = get_fannie_config_value(self.rattail_config, 'LANES')

        for i, lane in enumerate(lanes, 1):
            lane_data = dict(lane)
            lane_data['number'] = i
            del lane_data['user']
            del lane_data['pw']
            data.append(lane_data)

        return data


def defaults(config, **kwargs):
    base = globals()

    LaneView = kwargs.get('LaneView', base['LaneView'])
    LaneView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS scale item views
"""

from corepos.db.office_op import model as corepos

from .master import CoreOfficeMasterView


class ScaleItemView(CoreOfficeMasterView):
    """
    Base class for scale item views.
    """
    model_class = corepos.ScaleItem
    model_title = "CORE-POS Scale Item"
    url_prefix = '/core-pos/scale-items'
    route_prefix = 'corepos.scale_items'
    results_downloadable = True

    labels = {
        'plu': "PLU",
    }

    grid_columns = [
        'plu',
        'product',
        'price',
        'origin_text',
        'modified',
    ]

    def configure_grid(self, g):
        super(ScaleItemView, self).configure_grid(g)

        g.filters['plu'].default_active = True
        g.filters['plu'].default_verb = 'contains'

        g.set_type('price', 'currency')

        g.set_sort_defaults('plu')

        g.set_link('plu')
        g.set_link('product')

    def configure_form(self, f):
        super(ScaleItemView, self).configure_form(f)

        f.set_renderer('product', self.render_corepos_product)


def defaults(config, **kwargs):
    base = globals()

    ScaleItemView = kwargs.get('ScaleItemView', base['ScaleItemView'])
    ScaleItemView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE-POS product views
"""

from corepos.db.office_op import model as corepos
from corepos.db.util import table_exists

from webhelpers2.html import HTML, tags

from .master import CoreOfficeMasterView


class ProductView(CoreOfficeMasterView):
    """
    Base class for product views.
    """
    model_class = corepos.Product
    model_title = "CORE-POS Product"
    url_prefix = '/core-pos/products'
    route_prefix = 'corepos.products'
    results_downloadable = True

    labels = {
        'id': "ID",
        'upc': "UPC",
        'department_number': "Dept. No.",
        'foodstamp': "Food Stamp",
        'unit_of_measure': "Unit of Measure",
        'quantity_enforced': "Qty. Enforced",
        'id_enforced': "ID Enforced",
        'subdepartment_number': "Subdept. No.",
        'default_vendor_id': "Default Vendor ID",
        'current_origin_id': "Current Origin ID",
        'tax_rate_id': "Tax Rate ID",
    }

    grid_columns = [
        'upc',
        'brand',
        'description',
        'size',
        'department',
        'vendor',
        'normal_price',
        'scale',
        'cost',
    ]

    form_fields = [
        'upc',
        'id',
        'brand',
        'description',
        'size',
        'unit_of_measure',
        'formatted_name',
        'tare_weight',

        'department_number',
        'department',
        'subdepartment_number',
        'subdepartment',

        'id_enforced',

        'in_use',
        'quantity_enforced',
        'scale',
        'scale_price',
        'food_stamp',
        'wicable',
        'line_item_discountable',
        'flags',

        'price_method',
        'group_price',
        'quantity',
        'special_price_method',
        'special_group_price',
        'special_quantity',
        'tax_rate_id',
        'tax_rate',
        'mix_match_code',
        'discount_type',

        'default_vendor_id',
        'default_vendor',
        'cost',
        'vendor_items',

        'normal_price',
        'special_price',
        'start_date',
        'end_date',

        'created',
        'modified',

        'discount',

        'deposit',
        'local',

        'store_id',
        'current_origin_id',
        'last_sold',

        'scale_item',
        'user_info',
        'physical_location',
        'like_code',
    ]

    def configure_grid(self, g):
        super().configure_grid(g)

        g.set_joiner('department', lambda q: q.outerjoin(corepos.Department))
        g.set_sorter('department', corepos.Department.name)

        g.set_joiner('vendor', lambda q: q.outerjoin(corepos.Vendor,
                                                     corepos.Vendor.id == corepos.Product.default_vendor_id))
        g.set_sorter('vendor', corepos.Vendor.name)
        g.set_filter('vendor', corepos.Vendor.name)

        g.filters['upc'].default_active = True
        g.filters['upc'].default_verb = 'contains'

        g.filters['in_use'].default_active = True
        g.filters['in_use'].default_verb = 'is_true'

        g.set_type('cost', 'currency')
        g.set_type('normal_price', 'currency')

        g.set_sort_defaults('upc')

        g.set_link('upc')
        g.set_link('brand')
        g.set_link('description')

    def grid_extra_class(self, product, i):
        if not product.in_use:
            return 'warning'

    def get_instance_title(self, product):
        return "{} {}".format(product.upc, product.description)

    def configure_form(self, f):
        super().configure_form(f)

        if not table_exists(self.Session(), corepos.FloorSection):
            f.remove('physical_location')

        f.set_renderer('vendor', self.render_corepos_vendor)

        f.set_renderer('vendor_items', self.render_vendor_items)

        f.set_renderer('flags', self.render_flags)
        f.set_renderer('user_info', self.render_user_info)

        # TODO: these fields are not yet editable; improve later as needed
        f.set_readonly('department')
        f.set_readonly('subdepartment')
        f.set_readonly('tax_rate')
        f.set_readonly('physical_location')
        f.set_readonly('default_vendor')
        f.set_readonly('start_date')
        f.set_readonly('end_date')
        f.set_readonly('created')
        f.set_readonly('modified')
        f.set_readonly('last_sold')
        if self.editing:
            f.remove('vendor_items')
            f.remove('scale_item')
            f.remove('user_info')

        f.set_type('start_date', 'datetime_local')
        f.set_type('end_date', 'datetime_local')
        f.set_type('last_sold', 'datetime_local')
        f.set_type('modified', 'datetime_local')

        f.set_type('normal_price', 'currency')
        f.set_type('group_price', 'currency')
        f.set_type('special_price', 'currency')
        f.set_type('special_group_price', 'currency')
        f.set_type('cost', 'currency')
        f.set_type('deposit', 'currency')

    def render_flags(self, product, field):
        flags = product.flags
        if not flags:
            return ""

        # fetch all flags which are actually defined (supported)
        supported = {}
        for flag in self.Session.query(corepos.ProductFlag):
            supported[flag.bit_number] = flag

        # convert product's flag value to string of bits
        bflags = bin(flags)[2:]   # remove '0b' prefix
        bflags = reversed(bflags) # make bit #1 first in string, etc.

        # create list of items to show each "set" flag
        items = []
        for i, bit in enumerate(bflags, 1):
            if bit == '1':
                flag = supported.get(i)
                if flag:
                    items.append(HTML.tag('li', c=flag.description))
                else:
                    items.append(HTML.tag('li', c="(unsupported bit #{})".format(i)))

        return HTML.tag('ul', c=items)

    def render_user_info(self, product, field):
        user_info = product.user_info
        if not user_info:
            return ""
        text = str(user_info)
        url = self.request.route_url('corepos.products_user.view', upc=user_info.upc)
        return tags.link_to(text, url)

    def render_vendor_items(self, product, field):
        route_prefix = self.get_route_prefix()
        permission_prefix = self.get_permission_prefix()

        factory = self.get_grid_factory()
        g = factory(
            key='{}.vendor_items'.format(route_prefix),
            data=[],
            columns=[
                'vendor_name',
                'sku',
                'size',
                'cost_display',
                'units',
            ],
            labels={
                'sku': "SKU",
                'vendor_name': "Vendor",
                'cost_display': "Cost",
            },
        )
        return HTML.literal(
            g.render_buefy_table_element(data_prop='vendorItemsData'))

    def template_kwargs_view(self, **kwargs):
        kwargs = super().template_kwargs_view(**kwargs)
        product = kwargs['instance']
        app = self.get_rattail_app()

        vendor_items = []
        for item in product.vendor_items:
            vendor_items.append({
                'sku': item.sku,
                'vendor_name': item.vendor.name,
                'size': item.size,
                'cost_display': app.render_currency(item.cost),
                'units': item.units,
            })
        kwargs['vendor_items_data'] = vendor_items

        return kwargs

    def get_xref_buttons(self, product):
        app = self.get_rattail_app()
        corepos = app.get_corepos_handler()
        url = corepos.get_office_product_url(product.upc)
        if url:
            return [self.make_xref_button(url=url, text="View in CORE Office")]

    def download_results_fields_available(self, **kwargs):
        fields = super().download_results_fields_available(**kwargs)

        fields.append('superdepartment_number')

        return fields

    def download_results_setup(self, fields, progress=None):
        super().download_results_setup(fields, progress=progress)

        if 'superdepartment_number' in fields:
            mapping = {}
            super_departments = self.Session.query(corepos.SuperDepartment).all()
            for superdept in super_departments:
                if superdept.child_id in mapping:
                    if superdept.parent_id < mapping[superdept.child_id]:
                        mapping[superdept.child_id] = superdept.parent_id
                else:
                    mapping[superdept.child_id] = superdept.parent_id
            self.supermap = mapping

    def download_results_normalize(self, product, fields, **kwargs):
        data = super().download_results_normalize(product, fields, **kwargs)

        if 'superdepartment_number' in fields:
            data['superdepartment_number'] = None
            if product.department_number in self.supermap:
                data['superdepartment_number'] = self.supermap[product.department_number]

        return data


class ProductUserView(CoreOfficeMasterView):
    """
    Master view for `productUser` table
    """
    model_class = corepos.ProductUser
    model_title = "CORE-POS Product User"
    model_title_plural = "CORE-POS Products User"
    url_prefix = '/core-pos/products-user'
    route_prefix = 'corepos.products_user'

    labels = {
        'upc': "UPC",
    }

    grid_columns = [
        'upc',
        'description',
        'brand',
        'sizing',
        'enable_online',
        'sold_out',
    ]

    form_fields = [
        'upc',
        'product',
        'description',
        'brand',
        'sizing',
        'long_text',
        'photo',
        'enable_online',
        'sold_out',
        'sign_count',
    ]

    def configure_grid(self, g):
        super().configure_grid(g)

        g.filters['upc'].default_active = True
        g.filters['upc'].default_verb = 'contains'

        g.set_sort_defaults('upc')

        g.set_link('upc')
        g.set_link('description')

    def configure_form(self, f):
        super().configure_form(f)

        f.set_type('long_text', 'text')
        f.set_renderer('product', self.render_corepos_product)


class ProductFlagView(CoreOfficeMasterView):
    """
    Master view for product flags
    """
    model_class = corepos.ProductFlag
    model_title = "CORE-POS Product Flag"
    url_prefix = '/core-pos/product-flags'
    route_prefix = 'corepos.product_flags'

    def configure_grid(self, g):
        super().configure_grid(g)

        g.set_link('bit_number')
        g.set_link('description')

    def grid_extra_class(self, flag, i):
        if not flag.active:
            return 'warning'


def defaults(config, **kwargs):
    base = globals()

    ProductView = kwargs.get('ProductView', base['ProductView'])
    ProductView.defaults(config)

    ProductUserView = kwargs.get('ProductUserView', base['ProductUserView'])
    ProductUserView.defaults(config)

    ProductFlagView = kwargs.get('ProductFlagView', base['ProductFlagView'])
    ProductFlagView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Common menus for CORE-POS
"""


def make_corepos_menu(request):
    app = request.rattail_config.get_app()

    corepos_menu = {
        'title': "CORE-POS",
        'type': 'menu',
        'items': [
            {
                'title': "People",
                'type': 'menu',
                'items': [
                    {
                        'title': "Customers (classic)",
                        'route': 'corepos.customers',
                        'perm': 'corepos.customers.list',
                    },
                    {
                        'title': "Members (classic)",
                        'route': 'corepos.members',
                        'perm': 'corepos.members.list',
                    },
                    {
                        'title': "Customers (new)",
                        'route': 'corepos.customers_new',
                        'perm': 'corepos.customers_new.list',
                    },
                    {
                        'title': "Customer Accounts (new)",
                        'route': 'corepos.customer_accounts_new',
                        'perm': 'corepos.customer_accounts_new.list',
                    },
                    {
                        'title': "Customer Notifications",
                        'route': 'corepos.customer_notifications',
                        'perm': 'corepos.customer_notifications.list',
                    },
                    {
                        'title': "Suspensions",
                        'route': 'corepos.suspensions',
                        'perm': 'corepos.suspensions.list',
                    },
                    {
                        'title': "Member Types",
                        'route': 'corepos.member_types',
                        'perm': 'corepos.member_types.list',
                    },
                    {
                        'title': "Employees",
                        'route': 'corepos.employees',
                        'perm': 'corepos.employees.list',
                    },
                    {
                        'title': "Users",
                        'route': 'corepos.users',
                        'perm': 'corepos.users.list',
                    },
                    {
                        'title': "User Groups",
                        'route': 'corepos.user_groups',
                        'perm': 'corepos.user_groups.list',
                    },
                ],
            },
            {
                'title': "Products",
                'type': 'menu',
                'items': [
                    {
                        'title': "Products",
                        'route': 'corepos.products',
                        'perm': 'corepos.products.list',
                    },
                    {
                        'title': "Product Flags",
                        'route': 'corepos.product_flags',
                        'perm': 'corepos.product_flags.list',
                    },
                    {
                        'title': "Like Codes",
                        'route': 'corepos.like_codes',
                        'perm': 'corepos.like_codes.list',
                    },
                    {
                        'title': "Scale Items",
                        'route': 'corepos.scale_items',
                        'perm': 'corepos.scale_items.list',
                    },
                    {
                        'title': "Origins",
                        'route': 'corepos.origins',
                        'perm': 'corepos.origins.list',
                    },
                    {'type': 'sep'},
                    {
                        'title': "Super Departments",
                        'route': 'corepos.super_departments',
                        'perm': 'corepos.super_departments.list',
                    },
                    {
                        'title': "Departments",
                        'route': 'corepos.departments',
                        'perm': 'corepos.departments.list',
                    },
                    {
                        'title': "Subdepartments",
                        'route': 'corepos.subdepartments',
                        'perm': 'corepos.subdepartments.list',
                    },
                    {'type': 'sep'},
                    {
                        'title': "Batches",
                        'route': 'corepos.batches',
                        'perm': 'corepos.batches.list',
                    },
                    {
                        'title': "Batch Types",
                        'route': 'corepos.batch_types',
                        'perm': 'corepos.batch_types.list',
                    },
                ],
            },
            {
                'title': "Vendors",
                'type': 'menu',
                'items': [
                    {
                        'title': "Vendors",
                        'route': 'corepos.vendors',
                        'perm': 'corepos.vendors.list',
                    },
                    {
                        'title': "Vendor Items",
                        'route': 'corepos.vendor_items',
                        'perm': 'corepos.vendor_items.list',
                    },
                    {
                        'title': "Purchase Orders",
                        'route': 'corepos.purchase_orders',
                        'perm': 'corepos.purchase_orders.list',
                    },
                ],
            },
            {
                'title': "Transactions",
                'type': 'menu',
                'items': [
                    {
                        'title': "Transaction Details (current)",
                        'route': 'corepos.transaction_details',
                        'perm': 'corepos.transaction_details.list',
                    },
                    {
                        'title': "Transaction Details (archive)",
                        'route': 'corepos.transaction_details_archive',
                        'perm': 'corepos.transaction_details_archive.list',
                    },
                    {'type': 'sep'},
                    {
                        'title': "House Coupons",
                        'route': 'corepos.house_coupons',
                        'perm': 'corepos.house_coupons.list',
                    },
                    {
                        'title': "Stock Purchases",
                        'route': 'corepos.stock_purchases',
                        'perm': 'corepos.stock_purchases.list',
                    },
                    {
                        'title': "Tax Rates",
                        'route': 'corepos.taxrates',
                        'perm': 'corepos.taxrates.list',
                    },
                    {
                        'title': "Tenders",
                        'route': 'corepos.tenders',
                        'perm': 'corepos.tenders.list',
                    },
                ],
            },
            {
                'title': "Misc.",
                'type': 'menu',
                'items': [
                    {
                        'title': "Stores",
                        'route': 'corepos.stores',
                        'perm': 'corepos.stores.list',
                    },
                    {
                        'title': "Lanes",
                        'route': 'corepos.lanes',
                        'perm': 'corepos.lanes.list',
                    },
                    {
                        'title': "Parameters",
                        'route': 'corepos.parameters',
                        'perm': 'corepos.parameters.list',
                    },
                    {
                        'title': "Table Sync Rules",
                        'route': 'corepos.table_sync_rules',
                        'perm': 'corepos.table_sync_rules.list',
                    },
                ],
            },
        ],
    }

    corepos = app.get_corepos_handler()
    office_url = corepos.get_office_url()
    if office_url:
        corepos_menu['items'].insert(
            0, {
                'title': "Go to CORE Office",
                'url': '{}/'.format(office_url),
                'target': '_blank',
            })
        corepos_menu['items'].insert(
            1, {'type': 'sep'})

    return corepos_menu



# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2019 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Web Database Sessions

As with ``tailbone.db.Session``, the idea here is that we want to tie our
database sessions to the effective web "transaction" so that they are
auto-committed and -closed when the request has finished processing.
"""

from sqlalchemy.orm import sessionmaker, scoped_session
from zope.sqlalchemy import register


CoreOfficeSession = scoped_session(sessionmaker())
register(CoreOfficeSession)

CoreTransSession = scoped_session(sessionmaker())
register(CoreTransSession)

CoreTransArchiveSession = scoped_session(sessionmaker())
register(CoreTransArchiveSession)

# empty dict for now, this may be populated on app startup
ExtraCoreOfficeSessions = {}

# empty dict for now, this may be populated on app startup
ExtraCoreTransSessions = {}

# empty dict for now, this may be populated on app startup
ExtraCoreTransArchiveSessions = {}
